/* eventlog.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "session.h"
#include "eventlog.h"
#include "timestamp.h"
#include "main.h"
#include "misc.h"
#include "gui.h"
#include "scheduler.h"
#include "task.h"
#include "licon.xpm"

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((EventLog*)data)->hide(); return (TRUE); }
static void close_cb(GtkWidget *w, gpointer data)
{ ((EventLog*)data)->hide(); }

EventLog::EventLog(Main *m)
{
	GladeXML *gxml;

	main = m;

	gchar *fname = user_file("nomad.glade");
	gxml = glade_xml_new (fname, "eventlog");
	if (!gxml)
	{
		g_warning("Failed to load file nomad.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "eventlog");
	text1 = glade_xml_get_widget (gxml, "text1");
	button17 = glade_xml_get_widget (gxml, "button17");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 420, 280);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Event Log");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

	gtk_signal_connect (GTK_OBJECT (button17), "pressed",
		GTK_SIGNAL_FUNC (close_cb), this);
}

EventLog::~EventLog()
{
}

void EventLog::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void EventLog::hide()
{
	gtk_widget_hide(window);
}

void EventLog::update()
{
}

void EventLog::add(char *str)
{
	FILE *f;
	char fname[256];
	char buf[256];
	TimeStamp t;

	t.now();
	sprintf(buf, "%s: %s\n", t.date_to_str(1), str);
	gtk_text_insert(GTK_TEXT(text1), NULL, NULL, NULL, buf, -1);

	sprintf(fname, "%s/log", main->session->datadir);
	f = fopen(fname, "a");
	if (f)
	{
		fprintf(f, "%s", buf);
		fclose(f);
	}
}

