/* preferences.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gnome.h>
#include <gtk/gtkinvisible.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "preferences.h"
#include "main.h"
#include "session.h"
#include "gui.h"
#include "graphio.h"
#include "header.h"
#include "misc.h"
#include "licon.xpm"

static void select_community_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->select_community(row); }
static void unselect_community_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->unselect_community(); }
static void add_community_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->add_community(); }
static void del_community_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->del_community(); }
static void select_import_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->select_import(row); }
static void unselect_import_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->unselect_import(); }
static void add_import_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->add_import(); }
static void del_import_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->del_import(); }
static void ok_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->save();
	((Preferences*)data)->pref_copies();
	((Preferences*)data)->hide();
}
static void accept_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->save();
	((Preferences*)data)->pref_copies();
}
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->load(); ((Preferences*)data)->hide(); }
static void sidebar_button_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_button_pressed(w); }
static void sidebar_icon_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_icon_pressed(w); }
static void combo_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_combo(); }
static void test_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->notify_test(); }
static void import_import_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->import_import(); }

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Preferences*)data)->hide(); return (TRUE); }

Preferences::Preferences(Main *m)
{
	GladeXML *gxml;

	main = m;
	state = 0;
	community_row = -1;
	import_row = -1;

	gchar *fname = user_file("preferences.glade");
	gxml = glade_xml_new (fname, "preferences");
	if (!gxml)
	{
		g_warning("Failed to load file preferences.glade");
		return;
	}
	window = glade_xml_get_widget (gxml, "preferences");
	button17 = glade_xml_get_widget (gxml, "button17");
	button18 = glade_xml_get_widget (gxml, "button18");
	button19 = glade_xml_get_widget (gxml, "button19");
	sidebar = glade_xml_get_widget (gxml, "sidebar");
	notebook = glade_xml_get_widget (gxml, "notebook");
	notebook1 = glade_xml_get_widget (gxml, "notebook1");
	entry9 = glade_xml_get_widget (gxml, "entry9");
	entry10 = glade_xml_get_widget (gxml, "entry10");
	entry11 = glade_xml_get_widget (gxml, "entry11");
	entry13 = glade_xml_get_widget (gxml, "entry13");
//	entry22 = glade_xml_get_widget (gxml, "entry22");
//	entry23 = glade_xml_get_widget (gxml, "entry23");
	comboentry5 = glade_xml_get_widget (gxml, "comboentry5");
	fileentry2 = glade_xml_get_widget (gxml, "fileentry2");
	combo9 = glade_xml_get_widget (gxml, "combo9");
	combo10 = glade_xml_get_widget (gxml, "combo10");
	combo12 = glade_xml_get_widget (gxml, "combo12");
	combo13 = glade_xml_get_widget (gxml, "combo13");
	combo14 = glade_xml_get_widget (gxml, "combo14");
	combo15 = glade_xml_get_widget (gxml, "combo15");
	clist2 = glade_xml_get_widget (gxml, "clist2");
	clist3 = glade_xml_get_widget (gxml, "clist3");
	clist4 = glade_xml_get_widget (gxml, "clist4");
	clist5 = glade_xml_get_widget (gxml, "clist5");
	entry5 = glade_xml_get_widget (gxml, "entry5");
	entry6 = glade_xml_get_widget (gxml, "entry6");
	entry7 = glade_xml_get_widget (gxml, "entry7");
	entry8 = glade_xml_get_widget (gxml, "entry8");
	entry16 = glade_xml_get_widget (gxml, "entry16");
	entry17 = glade_xml_get_widget (gxml, "entry17");
	entry18 = glade_xml_get_widget (gxml, "entry18");
	entry19 = glade_xml_get_widget (gxml, "entry19");
	entry20 = glade_xml_get_widget (gxml, "entry20");
	entry21 = glade_xml_get_widget (gxml, "entry21");
	button7 = glade_xml_get_widget (gxml, "button7");
	button8 = glade_xml_get_widget (gxml, "button8");
	button9 = glade_xml_get_widget (gxml, "button9");
	button10 = glade_xml_get_widget (gxml, "button10");
	button13 = glade_xml_get_widget (gxml, "button13");
	button15 = glade_xml_get_widget (gxml, "button15");
	checkbutton1 = glade_xml_get_widget (gxml, "checkbutton1");
	checkbutton2 = glade_xml_get_widget (gxml, "checkbutton2");
	checkbutton3 = glade_xml_get_widget (gxml, "checkbutton3");
	checkbutton4 = glade_xml_get_widget (gxml, "checkbutton4");
	checkbutton5 = glade_xml_get_widget (gxml, "checkbutton5");
	checkbutton6 = glade_xml_get_widget (gxml, "checkbutton6");
	checkbutton7 = glade_xml_get_widget (gxml, "checkbutton7");
	checkbutton9 = glade_xml_get_widget (gxml, "checkbutton9");
	colorpicker2 = glade_xml_get_widget (gxml, "colorpicker2");
	colorpicker9 = glade_xml_get_widget (gxml, "colorpicker9");
	colorpicker6 = glade_xml_get_widget (gxml, "colorpicker6");
	colorpicker7 = glade_xml_get_widget (gxml, "colorpicker7");
	colorpicker14 = glade_xml_get_widget (gxml, "colorpicker14");
	colorpicker12 = glade_xml_get_widget (gxml, "colorpicker12");
	colorpicker10 = glade_xml_get_widget (gxml, "colorpicker10");
	colorpicker11 = glade_xml_get_widget (gxml, "colorpicker11");
	colorpicker13 = glade_xml_get_widget (gxml, "colorpicker13");
	colorpicker8 = glade_xml_get_widget (gxml, "colorpicker8");
	network_button = NULL;
	community_button = NULL;
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

  gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Preferences");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook1), FALSE);

	build_sidebar();

	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo13)->list), "selection_changed",
		GTK_SIGNAL_FUNC (combo_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist4), "select_row",
		GTK_SIGNAL_FUNC(select_community_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist4), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_community_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist5), "select_row",
		GTK_SIGNAL_FUNC(select_import_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist5), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_import_cb), this);
  gtk_signal_connect (GTK_OBJECT (entry5), "activate",
		GTK_SIGNAL_FUNC (add_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
		GTK_SIGNAL_FUNC (add_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
		GTK_SIGNAL_FUNC (del_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (button15), "clicked",
		GTK_SIGNAL_FUNC (test_cb), this);
  gtk_signal_connect (GTK_OBJECT (comboentry5), "activate",
		GTK_SIGNAL_FUNC (add_import_cb), this);
  gtk_signal_connect (GTK_OBJECT (button10), "clicked",
		GTK_SIGNAL_FUNC (add_import_cb), this);
  gtk_signal_connect (GTK_OBJECT (button9), "clicked",
		GTK_SIGNAL_FUNC (del_import_cb), this);
  gtk_signal_connect (GTK_OBJECT (button13), "clicked",
		GTK_SIGNAL_FUNC (import_import_cb), this);
  gtk_signal_connect(GTK_OBJECT(button17), "clicked",
		GTK_SIGNAL_FUNC(ok_cb), this);
  gtk_signal_connect(GTK_OBJECT(button18), "clicked",
		GTK_SIGNAL_FUNC(accept_cb), this);
  gtk_signal_connect(GTK_OBJECT(button19), "clicked",
		GTK_SIGNAL_FUNC(cancel_cb), this);

	load();		// load user preferences
	save();		// save to apply any new preferences

	unselect_community();
}

Preferences::~Preferences()
{
}

void Preferences::show()
{
	// only load if actually opening the window

	if (state==0) load();
	state = 1;
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Preferences::show_community()
{
	if (state==0) load();
	state = 1;

	if (network_button!=NULL)
		gtk_signal_emit_by_name (GTK_OBJECT (network_button), "clicked");
	if (community_button!=NULL)
		gtk_signal_emit_by_name (GTK_OBJECT (community_button), "clicked");

	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Preferences::hide()
{
	state = 0;
	gtk_widget_hide(window);
}

//////////////////////////////////////////////////////////////////////
// sidebar
//////////////////////////////////////////////////////////////////////

void Preferences::build_sidebar()
{
	GtkWidget *panel;

	panel = sidebar_button("Application");
	sidebar_icon(panel, "Session", "gnome-session.png", 4);
	sidebar_icon(panel, "Import", "grip-48.png", 6);
	sidebar_icon(panel, "Support", "default-applications.png", 5);
	sidebar_icon(panel, "Diagram", "gnome-graphics.png", 7);
	sidebar_pad(panel);
	panel = sidebar_button("Graphs");
	sidebar_icon(panel, "Variables", "gnome-monitor.png", 1);
	sidebar_icon(panel, "Sample Rate", "gnome-clock.png", 2);
	sidebar_pad(panel);
	panel = sidebar_button("Network");
	sidebar_icon(panel, "Community", "helix-setup-users3.png", 3);
	sidebar_icon(panel, "Notifier", "gnome-ticker.png", 8);
	sidebar_pad(panel);

	current_panel = NULL;
}

GtkWidget *Preferences::sidebar_button(char *label)
{
	GtkWidget *button, *button2, *panel;

	button = gtk_button_new_with_label (label);
	if (strcmp(label, "Network")==0) network_button = button;
	gtk_widget_show (button);
	panel = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), panel, TRUE, TRUE, 2);
	gtk_object_set_data (GTK_OBJECT (button), "panel", panel);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_button_pressed_cb), this);

	// icon buttons are in a radio group, we need to create a hidden
	// button to make it look like there are no buttons selected at start
	button_group = NULL;
	button2 = gtk_radio_button_new(button_group);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button2));
	gtk_box_pack_start (GTK_BOX (panel), button2, FALSE, TRUE, 0);
	gtk_button_clicked (GTK_BUTTON (button2));

	return panel;
}

void Preferences::sidebar_icon(GtkWidget *w, char *label, char *fname, int pg)
{
	GtkWidget *button, *vbox, *pixmap, *widget;
	gchar *str;

	button = gtk_radio_button_new (button_group);
	if (strcmp(label, "Community")==0) community_button = button;
	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
	gtk_object_set_data (GTK_OBJECT (button), "page", GINT_TO_POINTER(pg));
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_icon_pressed_cb), this);
	gtk_box_pack_start (GTK_BOX (w), button, FALSE, TRUE, 0);
	gtk_widget_show (button);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button), vbox);
	gtk_widget_show (vbox);

	str = user_file(fname);
	pixmap = gnome_pixmap_new_from_file(str);
	gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
	gtk_widget_show (pixmap);
	g_free(str);

	widget = gtk_label_new (label);
	gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
	gtk_label_set_line_wrap (GTK_LABEL (widget), FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
	gtk_widget_show (widget);
}

void Preferences::sidebar_pad(GtkWidget *w)
{
	GtkWidget *widget;

	widget = gtk_invisible_new();
	gtk_box_pack_end(GTK_BOX (w), widget, TRUE, TRUE, 0);
	gtk_widget_show(widget);
	gtk_object_set_data (GTK_OBJECT (w), "group", button_group);
}

void Preferences::sidebar_button_pressed(GtkWidget *w)
{
	GSList *grp;
	GtkWidget *panel;
	int page = 0;

	panel = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT (w), "panel"));
	if (current_panel != NULL)
	{
		gtk_widget_hide (current_panel);
	}
	if (current_panel != panel)
	{
		current_panel = panel;
		gtk_widget_show (panel);
		// set notebook page based on radio button currently set
		grp = (GSList*)(gtk_object_get_data(GTK_OBJECT (panel), "group"));
		while (grp!=NULL)
		{
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (grp->data)))
			{
				page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(grp->data), "page"));
			}
			grp = g_slist_next(grp);
		}
	}else
	{
		current_panel = NULL;
	}
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
}

void Preferences::sidebar_icon_pressed(GtkWidget *w)
{
	int page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "page"));
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
}


/***************************************************************************
 * Import External File List
 ***************************************************************************/

void Preferences::select_import(int r)
{
	import_row = r;
	gtk_widget_set_sensitive(button9, TRUE);

}

void Preferences::unselect_import()
{
	gtk_widget_set_sensitive(button9, FALSE);
	import_row = -1;
}

void Preferences::add_import()
{
	gchar *data[1];

	gtk_clist_freeze(GTK_CLIST(clist5));
	data[0] = strdup(gtk_entry_get_text(GTK_ENTRY(comboentry5)));
	if (strcmp(data[0], "")!=0)
		gtk_clist_append(GTK_CLIST(clist5), data);
	delete data[0];
	gtk_clist_set_sort_column (GTK_CLIST(clist5), 0);
	gtk_clist_sort (GTK_CLIST (clist5));
	gtk_clist_thaw(GTK_CLIST(clist5));
	gtk_entry_set_text(GTK_ENTRY(comboentry5), "");
}

void Preferences::del_import()
{
	gtk_clist_remove(GTK_CLIST(clist5), import_row);
}

void Preferences::set_config_from_import()
{
	gint i, num;
	gchar *data;
	gchar key[128];

	gnome_config_clean_section("/nomad/import");
	num = GTK_CLIST(clist5)->rows;
	for (i=0; i<num; i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist5), i, 0, &data);
		g_snprintf(key, 127, "/nomad/import/%d", i);
		gnome_config_set_string(key, data);
	}
}

void Preferences::set_import_from_config()
{
	gpointer itr;
	gchar *data[1];
	gchar *key, *val;
	gint flag = 0;

	gtk_clist_freeze(GTK_CLIST(clist5));
	gtk_clist_clear(GTK_CLIST(clist5));

	itr = gnome_config_init_iterator("/nomad/import");
	while (itr!=NULL)
	{
		itr = gnome_config_iterator_next(itr, &key, &val);
		if (val!=NULL)
		{
			data[0] = strdup(val);
			gtk_clist_append(GTK_CLIST(clist5), data);
			delete data[0];
			flag = 1;
		}
		g_free(key);
		g_free(val);
	}

	gtk_clist_set_sort_column (GTK_CLIST (clist5), 0);
	gtk_clist_sort (GTK_CLIST (clist5));
	gtk_clist_thaw(GTK_CLIST(clist5));
}

void Preferences::import_import()
{
	gint i, num;
	gchar *data;

	gtk_widget_set_sensitive(button13, FALSE);
	num = GTK_CLIST(clist5)->rows;
	for (i=0; i<num; i++)
	{
		data = NULL;
		gtk_clist_get_text(GTK_CLIST(clist5), i, 0, &data);
		if (data != NULL) main->session->import(data);
	}
	gtk_widget_set_sensitive(button13, TRUE);
}


////////////////////////////////////////////////////////////////////////////
/***************************************************************************
 * Community String List
 ***************************************************************************/
////////////////////////////////////////////////////////////////////////////

void Preferences::select_community(int r)
{
	community_row = r;
	gtk_widget_set_sensitive(button7, TRUE);

}

void Preferences::unselect_community()
{
	gtk_widget_set_sensitive(button7, FALSE);
	community_row = -1;
}

void Preferences::add_community()
{
	gchar *data[1];

	gtk_clist_freeze(GTK_CLIST(clist4));
	data[0] = strdup(gtk_entry_get_text(GTK_ENTRY(entry5)));
	if (strcmp(data[0], "")!=0)
		gtk_clist_append(GTK_CLIST(clist4), data);
	delete data[0];
	gtk_clist_set_sort_column (GTK_CLIST(clist4), 0);
	gtk_clist_sort (GTK_CLIST (clist4));
	gtk_clist_thaw(GTK_CLIST(clist4));
	gtk_entry_set_text(GTK_ENTRY(entry5), "");
}

void Preferences::del_community()
{
	gtk_clist_remove(GTK_CLIST(clist4), community_row);
}

void Preferences::set_config_from_community()
{
	gint i, num;
	gchar *data;
	gchar key[64];

	gnome_config_clean_section("/nomad/community");
	num = GTK_CLIST(clist4)->rows;
	for (i=0; i<num; i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist4), i, 0, &data);
		g_snprintf(key, 63, "/nomad/community/%d", i);
		gnome_config_set_string(key, data);
	}
}

void Preferences::set_community_from_config()
{
	gpointer itr;
	gchar *data[1];
	gchar *key, *val;
	gint flag = 0;

	gtk_clist_freeze(GTK_CLIST(clist4));
	gtk_clist_clear(GTK_CLIST(clist4));

	itr = gnome_config_init_iterator("/nomad/community");
	while (itr!=NULL)
	{
		itr = gnome_config_iterator_next(itr, &key, &val);
		if (val!=NULL)
		{
			data[0] = strdup(val);
			gtk_clist_append(GTK_CLIST(clist4), data);
			delete data[0];
			flag = 1;
		}
		g_free(key);
		g_free(val);
	}

	if (flag == 0)
	{
		data[0] = strdup("public");	// default
		gtk_clist_append(GTK_CLIST(clist4), data);
		delete data[0];
	}

	gtk_clist_set_sort_column (GTK_CLIST (clist4), 0);
	gtk_clist_sort (GTK_CLIST (clist4));
	gtk_clist_thaw(GTK_CLIST(clist4));
}

/////////////////////////////////////////////////////////////////
// Notifier Callbacks
/////////////////////////////////////////////////////////////////

void Preferences::notify_combo()
{
	gchar *nm;
	int i;

	nm = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo13)->entry));
	if (strcmp(nm, "Play Sound")==0) i=0; else
	if (strcmp(nm, "Send Email")==0) i=1; else
	if (strcmp(nm, "Call Pager")==0) i=2; else
	if (strcmp(nm, "Speak")==0) i=4; else i=3;
	gtk_notebook_set_page(GTK_NOTEBOOK (notebook1), i);
}

void Preferences::notify_test()
{
	char cmnd[128];
	gchar *nm, *str;

	nm = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo13)->entry));

	if (strcmp(nm, "Play Sound")==0)
	{
		str = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(fileentry2), TRUE);
		if (str!=NULL)
		{
			sprintf(cmnd, "playwave %s >/dev/null &", str);
			system(str);
		}
	}else
	if (strcmp(nm, "Send Email")==0)
	{
		str = gtk_entry_get_text(GTK_ENTRY(entry19));
		printf("Send email to \"%s\"\n", str);
		if (send_email(str, "NOMAD: Test Message", "This is a test")==0)
			printf("Failed to send email\n");
	}else
	if (strcmp(nm, "Call Pager")==0)
	{
		str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo15)->entry));
		printf("Use application %s to call pager on ", str);
		str = gtk_entry_get_text(GTK_ENTRY(entry20));
		printf("\"%s\"\n", str);
	}else
	if (strcmp(nm, "Speak")==0)
	{
		str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo14)->entry));
		printf("Speak message using application %s\n", str);
		system("say \"this is a test\" &");
	}else
	if (strcmp(nm, "Run External Program")==0)
	{
		str = gtk_entry_get_text(GTK_ENTRY(entry21));
		sprintf(cmnd, "%s &", str);
		system(str);
	}
}

/////////////////////////////////////////////////////////////////
// data
/////////////////////////////////////////////////////////////////

void Preferences::load()
{
	// Session
	set_editable_from_config(entry13, "/nomad/session/startup=");
	set_togglebutton_from_config(checkbutton1, "/nomad/session/auto_save_every=FALSE");
	set_time_from_config(entry8, entry16, "/nomad/session/save_session_time=2 Hours");
	set_togglebutton_from_config(checkbutton2, "/nomad/session/auto_save_on_quit=FALSE");
	set_togglebutton_from_config(checkbutton3, "/nomad/session/save_current_view=FALSE");
	set_time_from_config(entry6, entry17, "/nomad/session/ping_time=90 Seconds");
	set_time_from_config(entry7, entry18, "/nomad/session/topology_time=6 Hours");
	set_togglebutton_from_config(checkbutton9, "/nomad/session/topology_default=TRUE");
	set_togglebutton_from_config(checkbutton7, "/nomad/session/graph_default=TRUE");

	// Import
	set_import_from_config();

	// Support
	set_editable_from_config(entry9,  "/nomad/support/ping=xterm -e ping %s");
	set_editable_from_config(entry10, "/nomad/support/telnet=xterm -e telnet %s");
	set_editable_from_config(entry11, "/nomad/support/traceroute=xterm -e nomad-traceroute %s");

	// Diagram
	set_togglebutton_from_config(checkbutton3, "/nomad/diagram/show_leaf=TRUE");
	set_togglebutton_from_config(checkbutton3, "/nomad/diagram/show_empty_leaf=FALSE");
	set_editable_from_config(GTK_COMBO(combo9)->entry,  "/nomad/diagram/group_leaf=Bunched");
	set_editable_from_config(GTK_COMBO(combo12)->entry,  "/nomad/diagram/network_position=Automatic");
	set_togglebutton_from_config(checkbutton3, "/nomad/diagram/show_machines=FALSE");
	set_editable_from_config(GTK_COMBO(combo10)->entry,  "/nomad/diagram/group_machines=Bus");
	set_colour_from_config(colorpicker2,  "/nomad/diagram/background=#FFFFFF");
	set_colour_from_config(colorpicker9,  "/nomad/diagram/device_up=#00FF00");
	set_colour_from_config(colorpicker6,  "/nomad/diagram/device_down=#FF0000");
	set_colour_from_config(colorpicker14, "/nomad/diagram/device_up_selected=#007F00");
	set_colour_from_config(colorpicker7,  "/nomad/diagram/device_down_selected=#7F0000");
	set_colour_from_config(colorpicker12, "/nomad/diagram/network=#0000FF");
	set_colour_from_config(colorpicker10, "/nomad/diagram/network_empty=#00FFFF");
	set_colour_from_config(colorpicker11, "/nomad/diagram/network_selected=#00007F");
	set_colour_from_config(colorpicker13, "/nomad/diagram/machine=#7F00FF");
	set_colour_from_config(colorpicker8,  "/nomad/diagram/machine_selected=#7F007F");

	// Variables
	// Sample Rate
	set_graph_from_config();

	// Community
	set_community_from_config();

	// Notifier

	pref_copies();
}

void Preferences::save()
{
	// Session
	set_config_from_editable(entry13, "/nomad/session/startup");
	set_config_from_togglebutton(checkbutton1, "/nomad/session/auto_save_every");
	set_config_from_time(entry8, entry16, "/nomad/session/save_session_time");
	set_config_from_togglebutton(checkbutton2, "/nomad/session/auto_save_on_quit");
	set_config_from_togglebutton(checkbutton3, "/nomad/session/save_current_view");
	set_config_from_time(entry6, entry17, "/nomad/session/ping_time");
	set_config_from_time(entry7, entry18, "/nomad/session/topology_time");
	set_config_from_togglebutton(checkbutton9, "/nomad/session/topology_default");
	set_config_from_togglebutton(checkbutton7, "/nomad/session/graph_default");

	// Import
	set_config_from_import();

	// Support
	set_config_from_editable(entry9,  "/nomad/support/ping");
	set_config_from_editable(entry10, "/nomad/support/telnet");
	set_config_from_editable(entry11, "/nomad/support/traceroute");

	// Diagram
	set_config_from_togglebutton(checkbutton3, "/nomad/diagram/show_leaf");
	set_config_from_togglebutton(checkbutton3, "/nomad/diagram/show_empty_leaf");
	set_config_from_editable(GTK_COMBO(combo9)->entry,  "/nomad/diagram/group_leaf");
	set_config_from_editable(GTK_COMBO(combo12)->entry,  "/nomad/diagram/network_position");
	set_config_from_togglebutton(checkbutton3, "/nomad/diagram/show_machines");
	set_config_from_editable(GTK_COMBO(combo10)->entry,  "/nomad/diagram/group_machines");
	set_config_from_colour(colorpicker2,  "/nomad/diagram/background");
	set_config_from_colour(colorpicker9,  "/nomad/diagram/device_up");
	set_config_from_colour(colorpicker6,  "/nomad/diagram/device_down");
	set_config_from_colour(colorpicker14, "/nomad/diagram/device_up_selected");
	set_config_from_colour(colorpicker7,  "/nomad/diagram/device_down_selected");
	set_config_from_colour(colorpicker12, "/nomad/diagram/network");
	set_config_from_colour(colorpicker10, "/nomad/diagram/network_empty");
	set_config_from_colour(colorpicker11, "/nomad/diagram/network_selected");
	set_config_from_colour(colorpicker13, "/nomad/diagram/machine");
	set_config_from_colour(colorpicker8,  "/nomad/diagram/machine_selected");

	// Variables

	// Sample Rate

	// Community
	set_config_from_community();

	// Notifier


	gnome_config_sync();
}

// load values for local copy of frequently used variables
void Preferences::pref_copies()
{
	save_current_view =
		gnome_config_get_bool("/nomad/session/save_current_view=FALSE");
	main->gui->update_preferences();
}

void Preferences::set_graph_from_config()
{
	gchar *data[4], *datab[1];
	char buf[32], buf2[32], buf3[128];
	GraphIO gio;
	int i;

	gio.read_global();

	gtk_clist_freeze(GTK_CLIST(clist2));
	gtk_clist_clear(GTK_CLIST(clist2));
	for (i=0; i<gio.num_global_values; i++)
	{
		data[0] = strdup(gio.global_types[i].oid);
		data[1] = strdup(gio.global_types[i].label);
		data[2] = strdup("");
		data[3] = strdup("");
		gtk_clist_append(GTK_CLIST(clist2), data);
		delete data[0]; delete data[1];
		delete data[2]; delete data[3];
	}
	gtk_clist_thaw(GTK_CLIST(clist2));

	gtk_clist_freeze(GTK_CLIST(clist3));
	gtk_clist_clear(GTK_CLIST(clist3));
	for (i=0; i<gio.num_global_blocks; i++)
	{
		seconds_to_time_string(gio.global_blocks[i].gap, buf);
		seconds_to_time_string(gio.global_blocks[i].howlong, buf2);
		sprintf(buf3, "Every %s for %s", buf, buf2);
		datab[0] = strdup(buf3);
		gtk_clist_append(GTK_CLIST(clist3), datab);
		delete datab[0];
	}
	gtk_clist_thaw(GTK_CLIST(clist3));
}

gchar *get_community_list_string()
{
	gpointer itr;
	gchar *key, *val;
	gint flag = 0;
	gchar *str;

	itr = gnome_config_init_iterator("/nomad/community");
	while (itr!=NULL)
	{
		itr = gnome_config_iterator_next(itr, &key, &val);
		g_free(key);
		if (val!=NULL)
		{
			if (flag==0)
				str = g_strdup(val);
			else
			{
				key = g_strjoin(" ", str, val, NULL);
				g_free(str);
				str = key;
			}
			flag = 1;
		}
		g_free(val);
	}

	if (flag == 0)
	{
		str = g_strdup("public");
	}
	return str;
}

// Save & Load Routines

void set_editable_from_config(GtkWidget *w, gchar *config)
{
	gchar *str;

	str = gnome_config_get_string(config);
	gtk_entry_set_text(GTK_ENTRY(w), str);
	g_free(str);
}

void set_config_from_editable(GtkWidget *w, gchar *config)
{
	gchar *str;

	str = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	gnome_config_set_string(config, str);
	g_free(str);
}

void set_time_from_config(GtkWidget *w, GtkWidget *w2, gchar *config)
{
	gchar *str;
	gchar **sarray;

	str = gnome_config_get_string(config);
	sarray = g_strsplit(str, " ", 2);
	if (sarray[0]!=NULL) gtk_entry_set_text(GTK_ENTRY(w), sarray[0]);
	if (sarray[1]!=NULL) gtk_entry_set_text(GTK_ENTRY(w2), sarray[1]);
	g_strfreev (sarray);
}

void set_config_from_time(GtkWidget *w, GtkWidget *w2, gchar *config)
{
	gchar *str, *str2, *str3;

	str = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	str2 = gtk_editable_get_chars(GTK_EDITABLE(w2), 0, -1);
	str3 = g_strjoin(" ", str, str2, NULL);
	gnome_config_set_string(config, str3);
	g_free(str3);
	g_free(str2);
	g_free(str);
}

void set_config_from_colour(GtkWidget *w, gchar *config)
{
	guint8 r, g, b, a;
	gchar str[9];

	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(w), &r, &g, &b, &a);
	snprintf (str, 9, "#%02X%02X%02X", r, g, b);
	gnome_config_set_string (config, str);
}

void set_colour_from_config(GtkWidget *w, gchar *config)
{
	gchar *str;
	guint r, g, b;

	str = gnome_config_get_string(config);
	sscanf(str, "#%2X%2X%2X", &r, &g, &b);
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(w), r, g, b, 0);
	g_free(str);
}

void set_config_from_togglebutton(GtkWidget *w, gchar *config)
{
	gboolean value;

	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	gnome_config_set_bool(config, value);
}

void set_togglebutton_from_config (GtkWidget *w, gchar *config)
{
	gboolean value;

	value = gnome_config_get_bool (config);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), value);
}

