#include <string.h>
#include <stdio.h>
#include "timestamp.h"

TimeStamp::TimeStamp()
{
	empty();
}

TimeStamp::~TimeStamp()
{
}

TimeStamp::TimeStamp(TimeStamp &orig)
{
	ts = orig.ts;
}

void TimeStamp::operator = (const TimeStamp &orig)
{
	ts = orig.ts;
}

void TimeStamp::add(TimeStamp ts2)
{
	ts += ts2.ts;
}

void TimeStamp::empty()
{
	ts = 0;
}

void TimeStamp::now()
{
	ts = time(NULL);
}

// rewind the time to midnight lastnight
void TimeStamp::midnight()
{
	struct tm *tb;

	tb = gmtime(&ts);
	tb->tm_sec = 0;
	tb->tm_min = 0;
	tb->tm_hour = 0;
	ts = mktime(tb);
}

void TimeStamp::shift_days(int c)
{
	ts += (86400*c);
}

int TimeStamp::diff_days(TimeStamp t)
{
	return ((ts-t.ts)/86400);
}

void TimeStamp::setstring(char *buf)
{
	struct tm ptm;
	char year[8], month[8], day[8], hour[8], minute[8], second[8];
	char str[16];
	int i;

	if ((buf==NULL) || (strlen(buf)!=14)) return;

	for (i=0; i<4; i++) year[i] = buf[i];      year[4] = '\0';
	for (i=0; i<2; i++) month[i] = buf[i+4];   month[2] = '\0';
	for (i=0; i<2; i++) day[i] = buf[i+6];     day[2] = '\0';
	for (i=0; i<2; i++) hour[i] = buf[i+8];    hour[2] = '\0';
	for (i=0; i<2; i++) minute[i] = buf[i+10]; minute[2] = '\0';
	for (i=0; i<2; i++) second[i] = buf[i+12]; second[2] = '\0';

	sscanf(year, "%d", &i);                 ptm.tm_year = i-1900;
	sscanf(month, "%d", &i);                ptm.tm_mon = i-1;
	sscanf(day, "%d", &i);                  ptm.tm_mday = i;
	sscanf(hour, "%d", &i);                 ptm.tm_hour = i;
	sscanf(minute, "%d", &i);               ptm.tm_min = i;
	sscanf(second, "%d", &i);               ptm.tm_sec = i;
																					ptm.tm_isdst = 0;
	ts = mktime(&ptm);
}

char *TimeStamp::getstring()
{
	static char buf[16];
	struct tm *gmt;
	int d, m, y, h, n, s;

	gmt = gmtime(&ts);
	y = gmt->tm_year+1900;
	m = gmt->tm_mon+1;
	d = gmt->tm_mday;
	h = gmt->tm_hour;
	n = gmt->tm_min;
	s = gmt->tm_sec;
	sprintf(buf, "%04d%02d%02d%02d%02d%02d", y, m, d, h, n, s);
	return buf;
}

void TimeStamp::print()
{
	printf("%s", getstring());
}

time_t TimeStamp::compare(TimeStamp ts2)
{
	return (ts-ts2.ts);
}

int TimeStamp::zero()
{
	return (ts==0);
}

void TimeStamp::write_time(FILE *f)
{
	fputc((ts & 0xff000000)>>24, f);
	fputc((ts & 0xff0000)>>16, f);
	fputc((ts & 0xff00)>>8, f);
	fputc((ts & 0xff), f);
}

void TimeStamp::write_time64(FILE *f)
{
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc(0, f);
	fputc((ts & 0xff000000)>>24, f);
	fputc((ts & 0xff0000)>>16, f);
	fputc((ts & 0xff00)>>8, f);
	fputc((ts & 0xff), f);
}

void TimeStamp::read_time64(FILE *f)
{
	int a, b, c, d;
	// we throw away the first 4 bytes
//	fseek(f, 4L, SEEK_CUR);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	a = fgetc(f);
	b = fgetc(f);
	c = fgetc(f);
	d = fgetc(f);
	ts = (time_t) (((((a*256)+b)*256)+c)*256)+d;
}

int TimeStamp::get_day()
{
	struct tm *tb = gmtime(&ts);
	return (tb->tm_wday);
}

char *TimeStamp::date_to_str(int type)
{
	static char dout[32];
	struct tm *tb;
	char day[16];
	char ext[8];
	char month[16];
	
	tb = gmtime(&ts);

	if (tb->tm_wday == 0) strcpy(day, "Sun"); else
	if (tb->tm_wday == 1) strcpy(day, "Mon"); else
	if (tb->tm_wday == 2) strcpy(day, "Tue"); else
	if (tb->tm_wday == 3) strcpy(day, "Wed"); else
	if (tb->tm_wday == 4) strcpy(day, "Thu"); else
	if (tb->tm_wday == 5) strcpy(day, "Fri"); else
		strcpy(day, "Sat");

	if ((tb->tm_mday == 1) || (tb->tm_mday == 21) || (tb->tm_mday ==31))
		strcpy(ext, "st"); else
	if ((tb->tm_mday == 2) || (tb->tm_mday == 22))
		strcpy(ext, "nd"); else
	if ((tb->tm_mday == 3) || (tb->tm_mday == 23))
		strcpy(ext, "rd"); else strcpy(ext, "th");

	if (tb->tm_mon == 0) strcpy(month, "Jan"); else
	if (tb->tm_mon == 1) strcpy(month, "Feb"); else
	if (tb->tm_mon == 2) strcpy(month, "Mar"); else
	if (tb->tm_mon == 3) strcpy(month, "Apr"); else
	if (tb->tm_mon == 4) strcpy(month, "May"); else
	if (tb->tm_mon == 5) strcpy(month, "Jun"); else
	if (tb->tm_mon == 6) strcpy(month, "Jul"); else
	if (tb->tm_mon == 7) strcpy(month, "Aug"); else
	if (tb->tm_mon == 8) strcpy(month, "Sep"); else
	if (tb->tm_mon == 9) strcpy(month, "Oct"); else
	if (tb->tm_mon == 10) strcpy(month, "Nov"); else
	if (tb->tm_mon == 11) strcpy(month, "Dec");

	if (type==1)
		sprintf(dout, "%s %d%s %s %d %d:%02d:%02d", day, tb->tm_mday, ext, month, tb->tm_year+1900, tb->tm_hour, tb->tm_min, tb->tm_sec);
	else
	if (type==2)
		sprintf(dout, "%s %d%s %s %d", day, tb->tm_mday, ext, month, tb->tm_year+1900);
	else
	if (type==3)
		sprintf(dout, "%s %d%s %s", day, tb->tm_mday, ext, month);
	else
	if (type==4)
		sprintf(dout, "%s %d%s", day, tb->tm_mday, ext);
	else
	if (type==5)
		sprintf(dout, "%s", day);
	else
	if (type==6)
		sprintf(dout, "%c", day[0]);

	return dout;
}

